From 0a438f4139c1cf6a23c9b76b098f8629424062e7 Mon Sep 17 00:00:00 2001
From: Jarmo Torvinen <jarmo.torvinen@jutel.fi>
Date: Mon, 27 Apr 2009 14:01:44 +0300
Subject: [PATCH] support dynamic aspect ratio changes in x264 encoding

---
 modules/codec/x264.c |   50 ++++++++++++++++++++++++++++++++++++++++----------
 1 files changed, 40 insertions(+), 10 deletions(-)

diff --git a/modules/codec/x264.c b/modules/codec/x264.c
index 5004c2c..414f2a1 100644
--- a/modules/codec/x264.c
+++ b/modules/codec/x264.c
@@ -745,10 +745,33 @@ struct encoder_sys_t
     uint8_t         *p_buffer;
 
     mtime_t         i_interpolated_dts;
+    unsigned int    i_last_aspect_ratio;
 
     char *psz_stat_name;
 };
 
+
+/*****************************************************************************
+* SetAspectRatio: Set the aspect ratio of the encoder
+*****************************************************************************/
+static void SetAspectRatio(encoder_t *p_enc, encoder_sys_t *p_sys)
+{
+    int64_t i_num, i_den;
+    unsigned int i_dst_num, i_dst_den;
+
+    i_num = p_enc->fmt_in.video.i_aspect *
+        (int64_t)p_enc->fmt_in.video.i_height;
+    i_den = VOUT_ASPECT_FACTOR * p_enc->fmt_in.video.i_width;
+    vlc_ureduce( &i_dst_num, &i_dst_den, i_num, i_den, 0 );
+
+    p_sys->param.vui.i_sar_width = i_dst_num;
+    p_sys->param.vui.i_sar_height = i_dst_den;
+
+    msg_Dbg( p_enc, "aspect ratio is now: %d:%d", i_dst_num, i_dst_den);
+    p_sys->i_last_aspect_ratio = p_enc->fmt_in.video.i_aspect;
+}
+
+
 /*****************************************************************************
  * Open: probe the encoder
  *****************************************************************************/
@@ -1211,16 +1234,7 @@ static int  Open ( vlc_object_t *p_this )
 
     if( p_enc->fmt_in.video.i_aspect > 0 )
     {
-        int64_t i_num, i_den;
-        unsigned int i_dst_num, i_dst_den;
-
-        i_num = p_enc->fmt_in.video.i_aspect *
-            (int64_t)p_enc->fmt_in.video.i_height;
-        i_den = VOUT_ASPECT_FACTOR * p_enc->fmt_in.video.i_width;
-        vlc_ureduce( &i_dst_num, &i_dst_den, i_num, i_den, 0 );
-
-        p_sys->param.vui.i_sar_width = i_dst_num;
-        p_sys->param.vui.i_sar_height = i_dst_den;
+        SetAspectRatio(p_enc, p_sys);
     }
 
     if( p_enc->fmt_in.video.i_frame_rate_base > 0 )
@@ -1365,6 +1379,22 @@ static block_t *Encode( encoder_t *p_enc, picture_t *p_pict )
         pic.img.i_stride[i] = p_pict->p[i].i_pitch;
     }
 
+    if (p_sys->i_last_aspect_ratio != p_enc->fmt_in.video.i_aspect)
+    {
+        msg_Dbg( p_enc, "aspect ratio change detected: %d -> %d", 
+            p_sys->i_last_aspect_ratio, p_enc->fmt_in.video.i_aspect);
+
+        SetAspectRatio(p_enc, p_sys);
+        
+        // feed the new aspect ratio to x264 layer
+        x264_encoder_reconfig(p_sys->h, &p_sys->param);
+
+        // force an IDR frame, this persuades the x264 to
+        // send the aspect ratio immediately via SPS
+        // and not at next scene cut
+        pic.i_type = X264_TYPE_IDR;
+    }
+
 #if X264_BUILD >= 0x0013
     x264_encoder_encode( p_sys->h, &nal, &i_nal, &pic, &pic );
 #else
-- 
1.6.0.4

